# SEPDefs.py
# Bill Curry
# This file contains all of the build environment definitions needed for compiling.
# It also scans the source directory for .rs, .rt, .f90 and .f77 code.
# Broken up into parts called at the top of the SConstruct file and at the bottom.

import SCons
import glob
import string

def sep_defs(env):
    env['SEP']          =  '/usr/local/SEP'
    env['SEPBINDIR']    =  '$SEP/bin'
    env['SEPINCDIR']    =  '$SEP/include'
    env['SEPLIBDIR']    =  '$SEP/lib'
    env['INSTALL']      =  '/usr/bin/install -c -m 775'
    env['RESDIR']       =  '.'
    env['BINDIR']       =  '.'
    env['SRCDIR']       =  '.'
    env['OBJDIR']       =  '.'
    env['LIBDIR']       =  '$SEPLIBDIR'
    env['LIBPATH']      =  '$SEPINCDIR'

def c_defs(env):
    env['CC']           = 'cc'
    env['CCFLAGS']      = '-g -O2 '
    env['CPPFLAGS']     = '-DSOURCE='"'$SOURCE'"' -DLINUX'
    env['INCPREFIX']    = '-I'
    env['LINK']         = '$CC'
    env['LINKFLAGS']    = ''
    env['LIBLINKPREFIX']= ''

def f90_defs(env):
    env['FORTRAN']      = '/opt/intel/fc/9.0/bin/ifort'
    env['FORTRANPPOPT'] = ' -fpp3 -DSOURCE='"'$SOURCE'"' '
    env['FORTRANFLAGS'] = '$FORTRANPPOPT -g -FR -c -O -CB '
    env['INCPREFIX']    = '-I'
    env['LIBLINKPREFIX']= ''
    env['LINK']         = '$FORTRAN'
    env['LINKFLAGS']    = ''

def ratfor_defs(env):
    env['RATFOR']       = '/net/koko/bob/SEP_BOB/bin/ratfor90'
    env['RATFORFLAGS']  = '-sep -SOURCE $SOURCE'
    env['RATFORCOM']    = '$RATFOR <$SOURCE > $TARGET'
    env['RATFORSCOM']   = '$RATFOR $RATFORFLAGS <$SOURCE > $TARGET'

def loptran_defs(env):
    env['LOPTRAN']      = '/net/koko/bob/SEP_BOB/bin/ratfor90'
    env['LOPTRANFLAGS'] = '-sep '
    env['LOPTRANCOM']   = '$LOPTRAN $LOPTRANFLAGS <$SOURCE > $TARGET'

def seplib_defs(env):
    env.Replace(SEPLIB='-lsep')
    env.Replace(SEPLIBF='@SEPLIBF@')
    env.Replace(SEPLIBF90='-lsepf90 -lsep $F90LIBS')
    env.Replace(SEPLIB2DF90='-lsep2df90 -lsep3df90 -lsep3d -lsepf90 -lsep $F90LIBS')
    env.Replace(SEPLIB3D='-lsep3d -lsep')
    env.Replace(SEPLIB3DF='@SEPLIB3DF@')
    env.Replace(SEPLIB3DF90='-lsep3df90 -lsep3d -lsepf90 -lsep $F90LIBS')
    env.Replace(SUPERSET='-lsuperset -lsep3d -lsep')
    env.Replace(SUPERSETF='@SUPERSETF@')
    env.Replace(SUPERSETF90='-lsupersetf90 -lsuperset -lsep3df90 -lsep3d -lsepf90 -lsep $F90LIBS')
    env.Replace(VPLOT='-lvplot')
    env.Replace(VPLOTF='@VPLOTF@')
    env.Replace(VPLOTF90='-lvplotf90 -lvplot')
    env.Replace(GLPLOT='-lglplot')
    env.Replace(GEEF90='-lsepgeef90')
    env.Replace(GEELIB='-lsepgeef90 -lsep2df90 -lsep3df90 -lsep3d -lsepf90 -lsep $F90LIBS')
    env.Replace(SEPMATH='-lsepmath')
    env.Replace(SEPMATHF='@SEPMATHF@')
    env.Replace(SEPMATHF90='-lsepmathf90')
    env.Replace(SEPAUX='-lsepaux')
    env.Replace(SEPAUXF='@SEPAUXF@')
    env.Replace(SEPAUXF90='-lsepauxf90 -lsepaux')
    env.Replace(SEPFFT='-lsepfft -lsepfftf90 -lsepfft $SEPMATH')
    env.Replace(SEPFFTF='@SEPFFTF@')
    env.Replace(SEPFFTF90='-lsepfftf90 -lsepfft -lsepfftf90 -lsepfft -lsepauxf90 -lsepaux')
    env.Replace(SEPCONVERT='-lsepconvert')
    env.Replace(SEPCONVERTF='@SEPCONVERTF@')
    env.Replace(SEPCONVERTF90='-lsepconvertf90 -lsepconvert')
    env.Replace(SEPVECTOR='-lsepvector')
    env.Replace(SEPVECTORF='@SEPVECTORF@')
    env.Replace(SEPVECTORF90='-lsepvectorf90 -lsepvector')
    env.Replace(SEPFILTER='-lsepfilter')
    env.Replace(SEPTRAVEL='-lseptravel -lsepaux')
    env.Replace(SEPTRAVELF='@SEPTRAVELF@')
    env.Replace(SEPTRAVELF90='-lseptravelf90 -lseptravel')
    env.Replace(SEPVELANF='@SEPVELANF@')
    env.Replace(SEPVELANF90='-lsepvelanf90')
    env.Replace(SEPIMAGEF='@SEPIMAGEF@')
    env.Replace(SEPFILTERF='-lsepfilterf90 -lsepvectorf90 -lsepvector')
    env.Replace(SEPANISO='@SEPANISOF@')
    env.Replace(SUSEPLIB='@SUSEPLIB@')
    env.Replace(BEILIB90 = '-lbei90 $GEELIB')
    env.Replace(BEILIB='-lbei')
	#WHEN ADDED TO SEPlib need to be defined in configure.in
    env.Replace(TDFLIB = '-ltdf')
    env.Replace(BEILIB = '-lbei')
    env.Replace(PVILIB = '-lpvi')
    env.Replace(GEMLIBF = '-lgem')
    env.Replace(TDFLIBF = '-ltdf')
    env.Replace(BEILIBF = '-lbei')
    env.Replace(PVILIBF = '-lpvi')

#############################
#### Stuff for SEPbottom ####
#############################

def bottom_defs(env):
    env['FORTRANPATH']         = ['$SRCDIR']
    env['F90PATH']             = ['$SEPINCDIR','.','$SRCDIR']
    env['CPPPATH']             = ['$SEPINCDIR','.','$SRCDIR']

def find_src(env):
    env['PROGS']=[]
    env['F90PATH']             = ['$SEPINCDIR','.',env['SRCDIR']]
    env['LOPLIST']=get_files(env,'lop')
    env['F90LIST']=get_files(env,'f90')
    env['F77LIST']=get_files(env,'f77')
    env['RSLIST']=get_files(env,'rs')
    env['RTLIST']=get_files(env,'rt')
    env['RSTLIST']=get_files(env,'rst')
    env['R90LIST']=get_files(env,'r90')
    env['RS90LIST']=get_files(env,'rs90')
    env['CLIST']=get_files(env,'c')
    env['SRCLIST']=env['F90LIST']+env['F77LIST']+env['RSLIST']+env['RTLIST']+env['LOPLIST']+env['CLIST']+env['RSTLIST']+env['R90LIST']+env['RS90LIST']

def get_files(env,suffix):
    list=[]
    files= glob.glob(env['SRCDIR']+'/*'+'.'+suffix)
    for afile in files:
        filesplit = string.split(afile,'/')
        list.append(filesplit[-1])
    return list

    
