import os, stat, sys, types 
import re, string, urllib
import glob
import commands
# database for .sconsign
try:
    import dbhash
    db='dbhash'
except:
    try:
        import gdbm
        db='gdbm'
    except:
        db=None
        
try:
    import filecmp
except:
    import cmp


def FileExists(filename):
    return len(glob.glob(filename)) > 0

import SCons

# The following adds all SCons SConscript API to the globals of this module.
if SCons.__version__ == '0.96.90' or SCons.__version__ == '0.96.91':
    from SCons.Script import *
else:
    import SCons.Script.SConscript
    globals().update(SCons.Script.SConscript.BuildDefaultGlobals())

##############################################################################
# BEGIN CONFIGURATION VARIABLES
##############################################################################

sfsuffix = ''
vpsuffix = ''
sour=[]
scrp=[]
nodepend=[]

# path for binary files == DATAPATH
datapath = os.environ.get('DATAPATH')
if not datapath:
    try:
        file = open('./.datapath','r')
        #print file
    except:
        try:
            file = open(os.path.join(os.environ.get('HOME'),'.datapath'),'r')
        except:
            file = None
    if file:
        for line in file.readlines():
            #print line
            if line[0:5] == 'datap':
                new=string.split(line,'datapath=')
                new1=string.split(new[-1],'\n')
                datapath=new1[0]
            else:
                #check = re.match("(?:%s\s+)?datapath=(\S+)"
                #%os.uname()[1],line)
                machine=string.split(line," ")
                #print machine
                current=string.split(os.uname()[1],'.')
                #print current
                if machine[0]==current[0]:
                    new=string.split(line,'datapath=')
                    new1=string.split(new[-1],'\n')
                    datapath=new1[0]
            #if check:
            #    datapath = check.group(1)
        file.close()
    #print datapath
    if not datapath:
        datapath = './' # the ultimate fallback
dataserver = os.environ.get('RSF_DATASERVER','ftp://egl.beg.utexas.edu/')
tmpdatapath = os.environ.get('TMPDATAPATH',datapath)

# directory tree for executable files
top = os.environ.get('PYTHONPATH')
bindir = os.path.join(top,'bin')
libdir = os.path.join(top,'lib')
incdir = os.path.join(top,'include')
figdir = os.environ.get('FIGS',os.path.join(top,'figs'))

resdir = None

def set_dir(ref='.',dir=''):
     global resdir
     resdir = os.path.join(ref,dir)

set_dir()
# temporary (I hope)
sep = os.path.join(os.environ.get('SEP',''),'bin/')

#############################################################################
# CUSTOM BUILDERS
#############################################################################

def test(target=None,source=None,env=None):
    src = str(source[0])
    locked = re.sub('.*\/([^\/]+)\/([^\/]+)\/([^\/]+)\/Fig\/',
                    figdir+'/\\1/\\2/\\3/',os.path.abspath(src))
    print "Comparing %s and %s" % (locked,src)
    if os.path.isfile(locked):
        try:
            if not filecmp.cmp(locked,src,shallow=0):
                return 1
        except:
            if not cmp.cmp(locked,src):
                return 1
    else:
        print 'No locked file "%s" ' % locked
    return 0

View = Builder(action = sep + "xtpen $SOURCES",src_suffix=vpsuffix)
Test = Builder(action=Action(test))
#SourceSearch = Builder(action=Action(sourceSearch))
#ALLTARGS=[]

#############################################################################
# PLOTTING COMMANDS
#############################################################################

combine ={
    'SideBySideAniso': lambda n:
    sep + "vppen yscale=%d vpstyle=n gridnum=%d,1 $SOURCES" % (n,n),
    'OverUnderAniso': lambda n:
    sep + "vppen xscale=%d vpstyle=n gridnum=1,%d $SOURCES" % (n,n),
    'SideBySideIso': lambda n:
    sep + "vppen size=r vpstyle=n gridnum=%d,1 $SOURCES" % n,
    'OverUnderIso': lambda n:
    sep + "vppen size=r vpstyle=n gridnum=1,%d $SOURCES" % n,
    'TwoRows': lambda n:
    sep + "vppen size=r vpstyle=n gridnum=%d,2 $SOURCES" % (n/2),
    'Overlay': lambda n:
    sep + "vppen erase=o vpstyle=n $SOURCES",
    'Movie': lambda n:
    sep + "vppen vpstyle=n $SOURCES"
    }

#############################################################################
class Project(Environment):
    def __init__(self,**kw):
        apply(Environment.__init__,(self,),kw)
        self.EnsureSConsVersion(0,96)
#        opts = Options(os.path.join(libdir,'rsfconfig.py'))
#        rsfconf.options(opts)
#        opts.Add('TIMER','Whether to time execution')
#        opts.Update(self)
        cwd = os.getcwd()
        dir = os.path.basename(cwd)
        if datapath[:2] == './':
            self.path = datapath
        else:
            self.path = datapath+os.sep
            #self.path = datapath + dir + os.sep
        if not os.path.exists(self.path):
            os.mkdir(self.path)
        if db=='gdbm':
            self.SConsignFile(self.path+'.sconsign.'+db,gdbm)
        elif db=='dbhash':
            self.SConsignFile(self.path+'.sconsign.'+db,dbhash)
        else:
            self.SConsignFile(self.path+'.sconsign')
        self.resdir = resdir
        self.figdir = re.sub('.*\/((?:[^\/]+)\/(?:[^\/]+)\/(?:[^\/]+))$',
                             figdir+'/\\1',cwd)
        self.progsuffix = self['PROGSUFFIX']
        self.Append(ENV={'DATAPATH':self.path,
                         'TMPDATAPATH': tmpdatapath,
                         'PYTHONPATH': os.environ.get('PYTHONPATH',libdir), 
                         'XAUTHORITY':
                         os.path.join(os.environ.get('HOME'),'.Xauthority'),
                         'DISPLAY': os.environ.get('DISPLAY'),
                         'VPLOTFONTDIR': os.environ.get('VPLOTFONTDIR'),
                         'HOME': os.environ.get('HOME'),
                         'LD_LIBRARY_PATH': os.environ.get('LD_LIBRARY_PATH'),
                         'PATH' : os.environ.get('PATH'),
                         'PYTHONPATH':top},
                    BUILDERS={'View':View,
#                             #'Retrieve':Retrieve,
#                              'SourceSearch':SourceSearch,
                              'Test':Test},
                    LIBPATH=[libdir],
                    CPPPATH=[incdir],
                    LIBS=['sep','m'],
                    PROGSUFFIX='.exe')
        if sys.platform[:6] == 'cygwin':
            self['ENV']['PATH'] = self['ENV']['PATH'] + ':/usr/X11R6/bin'
            self['ENV']['SYSTEMROOT'] = os.environ.get('SYSTEMROOT')
        self['PROGPREFIX']=''
        self.view = []
#        self.sourceSearch = []
        self.lock = []
        self.test = []
        self.coms = []
        sys.path.append('../../../packages')
    def Exe(self,source,**kw):
        target = source.replace('.c','.x')
        return apply(self.Program,(target,source),kw)
########################################################################################################
    def SourceSearch(self,target1,source1,file1,suffix=vpsuffix,**kw):
        #flow=''
        again=False
        kw.update({'suffix':suffix})
        file = open(file1,'r')
        if file:
            for line in file.readlines():
                if line[:5]=='Plot(' or line[:5]=='Flow(':
                    tars=string.split(line,"'")
                    sour.append(tars[1])
                if line[:11]=='SConscript(':
                    script=string.split(line,"'")
                    scrp.append(script[1])
                    again=True
            if again==True:
                sc=scrp[0]
                del scrp[0]
                apply(self.SourceSearch,(target1,source1,sc,flow,kw))
            if len(scrp) > 0:
                sc=scrp[0]
                del scrp[0]
                apply(self.SourceSearch,(target1,source1,sc,flow,kw))
            if len(source1[0]) > 1:
                for p in range(0,len(source1)):
                    if (source1[p] in sour)==False:
                        nodepend.append(source1[p])
            else:
                if (source1 in sour)==False:
                    nodepend.append(source1)
            if len(nodepend) >= 1:
                done= 'No rule to build dependency '
                if len(nodepend[0]) > 1:
                       for q in range(0,len(nodepend)):
                           done=done+nodepend[q]+', '
                       print done
                else:
                    done=done+nodepend
                    print done
                sys.exit()
            if len(target[0]) > 1:
                for t in range(0,len(target)): 
                    commands.getstatusoutput('pstexpen '+target[t]+' '+target[t][:-1]+'ps;'+'epstopdf ' + target[t][:-1]+'ps')
        return None
#######################################################################################################    
    def Flow(self,target,source,flow,suffix=sfsuffix,src_suffix=sfsuffix,rsh=None):
        if not flow:
            return None        
        sources = []
        if source:
            if type(source) is types.ListType:
                files = source
            else:
                files = string.split(source)
            for file in files:
                if ('.' not in file):
                    file = file + src_suffix
                sources.append(file)
        else:
            source=0
        lines = string.split(flow,'&&')
        steps = []
        for line in lines:

            substeps = []
            sublines = string.split(line,'|')
            for subline in sublines:
                pars = string.split(subline)
                # command is assumed to be always first in line
                command = pars.pop(0)
                pars.insert(0,command)
                substeps.append(string.join(pars,' '))
            #<-
            steps.append(string.join(substeps," | "))
        #<- assemble the pipeline
        command = string.join(steps," && ")
        timer = self.get('TIMER')
        if timer and timer[0] != 'n' and timer[0] != '0':
            command = WhereIs('time') + ' ' + command
        targets = []
        if type(target) is types.ListType:
            files = target
        else:
            files = string.split(target)
        for file in files:
            if not re.search(suffix + '$',file):
                file = file + suffix
            targets.append(file)
        return self.Command(targets,sources,command)


    def Plot (self,target,source,flow=None,suffix=vpsuffix,vppen=None,
              view=None,**kw):
        if not flow: # two arguments
            flow = source
            source = target
        if combine.has_key(flow):
            if not type(source) is types.ListType:
                source = string.split(source)
            flow = apply(combine[flow],[len(source)])
            if vppen:
                flow = flow + ' ' + vppen
            kw.update({'src_suffix':vpsuffix})
        flowA = apply(self.Flow,(target,source,flow),kw)
        if target[-2:]=='.v':
            self.view.append(self.View(target + 'iew', flowA))
        kw.update({'suffix':suffix})
        return apply(self.Flow,(target,source,flow),kw)
 ##########################################################################################
    def ResultNR(self,target,source,flow=None,suffix=vpsuffix,**kw):
        if len(sys.argv) > 1:
            if sys.argv[1] == '-c':
                 case1 = 0
            else:
                case1 = 1
        else:
            case1 = 1
        flow = ''
        source = None
        if case1==1:
            if len(target[0]) > 1:
                for t in range(0,len(target)): 
                    if target[t][-2:]=='.v':
                        targs = string.split(target[t],'/')
                        if targs[-1][-5:]=='.view': 
                            Alias(targs[-1],'Fig/'+targs[-1])
                        commands.getstatusoutput('pstexpen '+target[t]+' '+target[t][:-1]+'ps;'+'epstopdf ' + target[t][:-1]+'ps')
                        if target[t][-3:]=='.ps':
                            commands.getstatusoutput('epstopdf ' + target[t])
            else:
                if target[-2:]=='.v':
                    targs = string.split(target,'/')
                    if targs[-1][-5:]=='.view': 
                        Alias(targs[-1],'Fig/'+targs[-1])
                    commands.getstatusoutput('pstexpen '+target+' '+target[:-1]+'ps;'+'epstopdf ' + target[:-1]+'ps')
                if target[-3:]=='.ps':
                    commands.getstatusoutput('epstopdf ' + target)    
        else:
            return None
 ##########################################################################################
    def ResultER(self,target,source,flow=None,suffix=vpsuffix,**kw):
        if len(sys.argv) > 1:
            if sys.argv[1] == '-c':
                 case1 = 0
            else:
                case1 = 1
        else:
            case1 = 1
        #case1=1
        if case1==1:
            t=0
            if COMMAND_LINE_TARGETS==[]:
                t=1
            elif len(COMMAND_LINE_TARGETS[0]) > 1:
                t=1
            if t==1:
                for i in range(0, len(COMMAND_LINE_TARGETS)):
                    targs = string.split(COMMAND_LINE_TARGETS[i],'/')
                    if targs[-1][-5:]=='.view':
                        Alias(targs[-1],'Fig/'+targs[-1])
            if not flow: # two arguments
                flow = source
                source = target
            if combine.has_key(flow):
                if not type(source) is types.ListType:
                    source = string.split(source)
                flow = apply(combine[flow],[len(source)])
                if vppen:
                    flow = flow + ' ' + vppen
                kw.update({'src_suffix':vpsuffix})
            if (target[-3:]=='.v3'):
                return apply(self.Plot,(target,source,flow),kw)
            else:
                if len(target[0]) > 1:
                    for t in range(0,len(target)):
                        flow = flow + ';chmod a-x '+ target[t]
                        flow = flow + '; pstexpen '+ target[t]+ ' '+target[t][:-1]+'ps'
                        flow = flow + '; epstopdf ' + target[t][:-1]+'ps'
                else:
                    flow = flow + ';chmod a-x '+ target
                    flow = flow + '; pstexpen '+ target+ ' '+target[:-1]+'ps'
                    flow = flow + '; epstopdf ' + target[:-1]+'ps'
                    plot = apply(self.Plot,(target,source,flow),kw)
                return plot
        else:
            return None
    ##########################################################
    def ResultCR(self,target,source,flow=None,suffix=vpsuffix,**kw):
        case1=1
        if len(sys.argv) > 1:
            for m in range(0,len(sys.argv)):
                if sys.argv[m] == '-c':
                    case1 = 0
        else:
            case1 = 1
        if case1==1:
            t=0
            if COMMAND_LINE_TARGETS==[]:
                t=1
            elif len(COMMAND_LINE_TARGETS[0]) > 1:
                t=1
            if t==1:
                for i in range(0, len(COMMAND_LINE_TARGETS)):
                    targs = string.split(COMMAND_LINE_TARGETS[i],'/')
                    if targs[-1][-5:]=='.view':
                        Alias(targs[-1],'Fig/'+targs[-1])
            if FileExists(target)==False:
                if not flow: # two arguments
                    flow = source
                    source = target
                if combine.has_key(flow):
                    if not type(source) is types.ListType:
                        source = string.split(source)
                    flow = apply(combine[flow],[len(source)])
                    if vppen:
                        flow = flow + ' ' + vppen
                    kw.update({'src_suffix':vpsuffix})
                flow = flow + ';chmod a-x '+ target
                flow = flow + '; pstexpen '+ target+ ' '+target[:-1]+'ps'
                flow = flow + '; epstopdf ' + target[:-1]+'ps'
                plot = apply(self.Plot,(target,source,flow),kw)
                return plot
            else:
                ss='SConstruct'
                flow=''
                return apply(self.SourceSearch,(target,source,ss),kw)
#############################################################################3
    def End(self):
        if self.view: # if any results
            self.Alias('view',self.view)
            self.Alias('lock',self.lock)
            self.Alias('test',self.test)
        self.Command('.sf_uses',None,'echo %s' % string.join(self.coms,' '))

# Default project
project = Project()
def Flow(target,source,flow,**kw):
    return apply(project.Flow,(target,source,flow),kw)
def Plot (target,source,flow=None,**kw):
    return apply(project.Plot,(target,source,flow),kw)
def ResultER(target,source,flow=None,**kw):
    return apply(project.ResultER,(target,source,flow),kw)
def ResultCR(target,source,flow=None,**kw):
    return apply(project.ResultCR,(target,source,flow),kw)
def ResultNR(target,source,flow=None,**kw):
    return apply(project.ResultNR,(target,source,flow),kw)
def SourceSearch(source,file1=None,**kw):
    return apply(project.SourceSearch,(source,file1),kw)
def Exe(source,**kw):
    return apply(project.Exe,[source],kw)
def End(**kw):
    return apply(project.End,[],kw)
def Program(*arg,**kw):
    return apply(project.Program,arg,kw)
def Get(name):
    return project['ENV'].get(name)
def Program90(prog):
    sources = [prog]
#$    rsfconf.depends90(project,sources,prog)
#$    return project.Program(prog,
#                           map(lambda x: x + '.f90',sources),
#                           F90PATH=[incdir],
#                           LIBS=['rsff90','rsf','m'],
#                           LINK=project.get('F90'),
#                           LINKFLAGS=project.get('F90FLAGS'))

if __name__ == "__main__":
     import pydoc
     pydoc.help(Project)
     
#   $Id: rsfproj.py 1461 2005-09-25 02:27:12Z fomels $
