# SEPProg.py
# This file contains builders for .rs and .rt files.
# It also contains the scanner for .f90 files and creates build rules for all .rs, .rt, .f90 and .f77 code
import SCons
import re
import os
import string
# Top part

def createRSRTFileBuilders(env):
    RFScan = SCons.Scanner.Fortran.FortranScan(env['SRCDIR'])
    SCons.Defaults.ObjSourceScan.add_scanner('.rs', RFScan)
    SCons.Defaults.ObjSourceScan.add_scanner('.rt', RFScan)
    env['SCANNERS']=RFScan
    env.Append(FORTRANSUFFIXES = ['.rt','.rs','.r','.rs90','.rst'])
    try:
        rsbuilder = env['BUILDERS']['RSFile']
    except KeyError:
        rsbuilder = SCons.Builder.Builder(action='$RATFORSCOM',
                                          src_suffix='.rs',
                                          src_scanner=RFScan,
                                          suffix='.f90')
        env['BUILDERS']['RSFile'] = rsbuilder
        env['RSFILESUFFIX'] = '.rs'
    try:
        rtbuilder = env['BUILDERS']['RTFile']
    except KeyError:
        rtbuilder = SCons.Builder.Builder(action='$RATFORCOM',
                                          src_suffix='.rt',
                                          src_scanner=RFScan,
                                          suffix='.f90')
        env['BUILDERS']['RTFile'] = rtbuilder
        env['RTFILESUFFIX'] = '.rt'
    return(rsbuilder,rtbuilder)

def createLOPFileBuilder(env):
    LOPScan = SCons.Scanner.Fortran.FortranScan(env['SRCDIR'])
    SCons.Defaults.ObjSourceScan.add_scanner('.lop', LOPScan)
    env['SCANNERS']=LOPScan
    env.Append(FORTRANSUFFIXES = ['.lop'])
    try:
        lopbuilder = env['BUILDERS']['LOPFile']
    except KeyError:
        lopbuilder = SCons.Builder.Builder(action='$LOPTRANCOM',
                                          src_suffix='.lop',
                                          src_scanner=LOPScan,
                                          suffix='.f90')
        env['BUILDERS']['LOPFile'] = lopbuilder
        env['LOPFILESUFFIX'] = '.lop'
    return(lopbuilder)

# Bottom part

def generate_rules(env,objects,programs):
    generate_rules_ratlop(env,'LOPLIST','lop',objects,programs,depend_suffixes=['lop','f90'])
    generate_rules_ratlop(env,'RTLIST','rt',objects,programs,depend_suffixes=['rt','f90'])
    generate_rules_ratlop(env,'RSLIST','rs',objects,programs,depend_suffixes=['rt','f90'])
    generate_rules_ratlop(env,'RSTLIST','rst',objects,programs,depend_suffixes=['rst','rt','f90'])
    generate_rules_ratlop(env,'R90LIST','r90',objects,programs,depend_suffixes=['r90','f90'])
    generate_rules_ratlop(env,'RS90LIST','rs90',objects,programs,depend_suffixes=['rt','rst','f90'])
    generate_rules_normal(env,'F90LIST','f90',objects,programs,depend_suffixes=['f90','lop','r90','rt','r'],avoid_suffixes=['rs','rt','r','r90','rs90'])
    generate_rules_normal(env,'F77LIST','f77',objects,programs,depend_suffixes=['f77','r'],avoid_suffixes=['rs','rt','r'])
    generate_rules_normal(env,'CLIST','c',objects,programs,depend_suffixes=['c'],avoid_suffixes=[])

def generate_rules_normal(env,list,suffix,objects,programs,depend_suffixes,avoid_suffixes):
    sl=-len(suffix)-1                                                        # to strip off the suffix
    if (len(env[list])>0 and objects=="yes"):                                # see if we need to make rules
        for afile in env[list]:                                              # loop over files in list
            uniquerule=1                                                     # see if a ratfor file exists already and we're duplicating stuff
            for abadsuffix in avoid_suffixes:
                if afile[0:sl]+abadsuffix in env['SRCLIST']:
                    uniquerule=0
            if uniquerule>0:                                                 # if that is not the case
                env.Object(target=env['OBJDIR']+'/'+afile[0:sl]+'.o',source=env['SRCDIR']+'/'+afile)
                listall=[]
                for dsuff in depend_suffixes:                                # find all dependencies in source directory
                    listtemp=[]
                    SConsDepend(env,listtemp,afile[0:sl]+'.'+suffix,'.'+dsuff)
                    listall=listall+listtemp
                objlist=[]
                for thing in listall:                                        # add those to the object list and make the dependency 
                    objlist.append(env['OBJDIR']+'/'+thing+'.o')
                    env.Depends(afile[0:sl]+'.o',env['OBJDIR']+'/'+thing+'.o')
                if (programs=="yes" and IsProgram(env,afile)>1):             # check to see if we need to make a program rule
                    print 'Program '+afile[0:sl]+' depends on:'+' '.join(objlist)
                    env.Program(target=env['BINDIR']+'/'+afile[0:sl]+'.x',source=[objlist,env['OBJDIR']+'/'+afile[0:sl]+'.o'],LIBS=env['libs'],LIBPATH=env['libpath'])

def generate_rules_ratlop(env,list,suffix,objects,programs,depend_suffixes):
    sl=-len(suffix)-1                                                        # to strip off the suffix
    if (len(env[list])>0):                                                   # see if we need to make rules
        for afile in env[list]:                                              # loop over files in list
            if (suffix==('rs' or 'rst' or 'RS' or 'rs90')):
                env.RSFile(target=env['SRCDIR']+'/'+afile[0:sl]+'.f90',source=env['SRCDIR']+'/'+afile)
            elif (suffix==('rt' or 'RT' or 'r90')):
                env.RTFile(target=env['SRCDIR']+'/'+afile[0:sl]+'.f90',source=env['SRCDIR']+'/'+afile)
            elif (suffix=='lop'):
                env.LOPFile(target=env['SRCDIR']+'/'+afile[0:sl]+'.f90',source=env['SRCDIR']+'/'+afile)
            if (objects=='yes'):
                env.Object(target=env['OBJDIR']+'/'+afile[0:sl]+'.o',source=env['SRCDIR']+'/'+afile[0:sl]+'.f90')
                listall=[]
                for dsuff in depend_suffixes:                                # find all dependencies in source directory
                    listtemp=[]
                    SConsDepend(env,listtemp,afile[0:sl]+'.'+suffix,dsuff)
                    listall=listall+listtemp
                objlist=[]
                for thing in listall:                                        # add those to the object list and make the dependency 
                    objlist.append(env['OBJDIR']+'/'+thing+'.o')
                    env.Depends(afile[0:sl]+'.o',env['OBJDIR']+'/'+thing+'.o')
                if (programs=="yes" and IsProgram(env,afile)>1 and suffix==('rs' or 'rst' or 'RS' or 'rs90')):             # check to see if we need to make a program rule
                    print 'Program '+afile[0:sl]+'.x depends on: '+' '.join(objlist)
                    env.Program(target=env['BINDIR']+'/'+afile[0:sl]+'.x',source=[objlist,env['OBJDIR']+'/'+afile[0:sl]+'.o'],LIBS=env['libs'],LIBPATH=env['libpath'])

def DumpEnv( env, key = None, header = None, footer = None ):
    """
    Using the standard Python pretty printer, dump the contents of the
    scons build environment to stdout.

    If the key passed in is anything other than 'env', then that will
    be used as an index into the build environment dictionary and
    whatever is found there will be fed into the pretty printer. Note
    that this key is case sensitive.

    The header and footer are simple mechanisms to allow printing a
    prefix and suffix to the contents that are dumped out. They are
    handy when using DumpEnv to dump multiple portions of the
    environment.
    """
    import pprint
    pp = pprint.PrettyPrinter( indent = 2 )
    if key:
        dict = env.Dictionary( key )
    else:
        dict = env.Dictionary()
    if header:
        print header
    pp.pprint( dict )
    if footer:
        print footer

def IsProgram(env,file):
    progcount=0
    program_regex = re.compile('program', re.IGNORECASE)
    if os.path.exists(env['SRCDIR']+'/'+file):
        fd = open(env['SRCDIR']+'/'+file,'r')
        for line in fd.readlines():
            if program_regex.search(line) is not None:
                progcount=progcount+1
        fd.close()
    return progcount

def SConsDepend(env,list,file,ext):
    use_regex = re.compile("(?i)(?:^|;)\s*USE(?:\s+|(?:(?:\s*,\s*(?:NON_)?INTRINSIC)?\s*::))\s*(\w+)")
    include_regex = re.compile("""(?i)(?:^|['">]\s*;)\s*INCLUDE\s+(?:\w+_)?[<"'](.+?)(?=["'>])""")
    if os.path.exists(env['SRCDIR']+'/'+file):
        fd = open(env['SRCDIR']+'/'+file,'r')
        for line in fd.readlines():
            for inc in use_regex.findall(line):
                if inc not in list and os.path.exists(env['SRCDIR']+'/'+inc+ext):
                    list.append(inc)
                    SConsDepend(env,list,inc+ext,ext)
        fd.close()
