function newcnstr( prob, x, y, op, cheat )
error( nargchk( 4, 5, nargin ) );
if nargin < 5, cheat = false; end

%
% Check problem
%

if ~isa( prob, 'cvxprob' ),
    error( 'First argument must be a cvxprob object.' );
end
global cvx___
p = index( prob );

%
% Check arguments
%

cx = isnumeric( x ) | isa( x, 'cvx' );
cy = isnumeric( y ) | isa( y, 'cvx' );
if ~cx,
    x = cvx_collapse( x, false, true );
    cx = isnumeric( x ) | isa( x, 'cvx' );
end
if ~cy,
    y = cvx_collapse( y, false, true );
    cy = isnumeric( y ) | isa( y, 'cvx' );
end
sx = size( x );
sy = size( y );
if ~cx | ~cy,
    if cx | cy,
        error( sprintf( 'Invalid CVX constraint: {%s} %s {%s}', class( x ), op, class( y ) ) );
    elseif ~isequal( sx, sy ),
        error( 'The left- and right-hand sides have incompatible sizes.' );
    else
        nx = prod( sx );
        for k = 1 : nx,
            newcnstr( prob, x{k}, y{k}, op );
        end
        return
    end
end

%
% Check readlevel
%

tx = cvx_readlevel( x );
tx = any( tx( : ) > p );
ty = cvx_readlevel( y );
ty = any( ty( : ) > p );
if tx | ty,
    error( 'Constraints may not involve internal, read-only variables.' );
end

%
% Check for a dual reference
%

if isa( x, 'cvx' ),
    dx = getdual( x );
else
    dx = '';
end
if isa( y, 'cvx' ),
    dy = getdual( y );
else
    dy = '';
end
if isempty( dx ),
    dx = dy;
elseif ~isempty( dy ),
    error( [ 'Two dual variables found: "', dx, '","', dy, '"' ] );
end
if ~isempty( dx ),
    duals = cvx___.problems( p ).duals;
    try
        dual = builtin( 'subsref', duals, dx );
    catch
        nm = cvx_subs2str( dx );
        error( [ 'Dual variable "', nm(2:end), '" has not been declared.' ] );
    end
    if ~isempty( dual ),
        nm = cvx_subs2str( dx );
        error( [ 'Dual variable "', nm(2:end), '" already in use.' ] );
    end
end

%
% Handle the SDP case
%

sdp_mode = false;
mx = sx( 1 ) > 1 & sx( 2 ) > 1;
my = sy( 1 ) > 1 & sy( 2 ) > 1;
if op(1) ~= '=' & cvx___.problems( p ).sdp & ( mx | my ),

    if sx( 1 ) ~= sx( 2 ) | sy( 1 ) ~= sy( 2 ),
        error( 'SDP constraints must be square.' );
    elseif ~isequal( sx, sy ) & ( mx | cvx_isnonzero( x ) ) & ( my | cvx_isnonzero( y ) ),
        error( 'Left- and right-hand sides have incompatible sizes.' );
    elseif ~cvx_isaffine( x ) | ~cvx_isaffine( y ),
        error( 'Both sides of an SDP constraint must be affine.' );
    elseif op( 1 ) == '>',
        x = minus( x, y, cheat );
        y = semidefinite( size( x ), ~isreal( x ) );
        sz = size( x );
    else
        y = minus( y, x, cheat );
        x = semidefinite( size( y ), ~isreal( y ) );
        sz = size( y );
    end
    op = '==';
    sdp_mode = true;

elseif ~cheat,

    persistent map_eq map_le map_ge map_ne
    if isempty( map_ge ),
        map_ne  = cvx_remap( 'constant' );
        map_ne  = map_ne' * map_ne;
        temp    = ~map_ne;

        map_eq2 = cvx_remap( 'log-affine' );
        map_eq2 = ( map_eq2' * map_eq2 ) & temp;
        map_eq3 = cvx_remap( 'affine' );
        map_eq3 = map_eq3' * map_eq3;
        map_eq  = 2 * map_eq2 + map_eq3;

        % Trivially feasible constraints
        map_ge1 = ( cvx_remap( 'log-concave' )' * cvx_remap( 'nonpositive' ) ) & temp;
        % Trivially infeasible constraints
        map_ge2 = ( cvx_remap( 'nonpositive' )' * cvx_remap( 'log-convex'  ) ) & temp;
        % Full geometric constraints
        map_ge3 = ( cvx_remap( 'log-concave' )' * cvx_remap( 'log-convex'  ) ) & temp;
        % Linear constraints
        map_ge4 = ( cvx_remap( 'concave' )' * cvx_remap( 'convex' ) ) & ~map_ge3;
        map_ge = map_ge4 + 2 * map_ge3 + 3 * map_ge2 + 4 * map_ge1;

        map_le = map_ge';
    end
    switch op(1),
        case '<',
            remap = map_le;
        case '>',
            remap = map_ge;
        case '~',
            remap = map_ne;
        otherwise,
            remap = map_eq;
    end
    vx = cvx_classify( x );
    vy = cvx_classify( y );
    vm = vx + size( remap, 1 ) * ( vy - 1 );
    vr = remap( vm );
    tt = vr( : ) == 0;
    if any( tt ),
        [ vu, vi ] = unique( vm );
        vi = vi( remap( vu ) == 0 );
        strs = {};
        xt = x; yt = y;
        for k = 1 : length( vi ),
            if any( sx ~= 1 ), xt = x(vi(k)); end
            if any( sy ~= 1 ), yt = y(vi(k)); end
            strs{k+1} = sprintf( '   Invalid constraint: {%s} %s {%s}\n', cvx_class( xt, true, false ), op, cvx_class( yt, true, false ) );
        end
        error( [ sprintf( 'Disciplined convex programming error:\n' ), strs{:} ] );
    end
    tt = vr( : ) == 2;
    tt = tt( : );
    if any( tt ),
        if all( tt ),
            x = log( x );
            y = log( y );
        else
            x( tt ) = log( x( tt ) );
            y( tt ) = log( y( tt ) );
        end
    end
    tt = vr( : ) == 3;
    tt = tt( : );
    if any( tt ),
        x( tt ) = 0;
        y( tt ) = 1 - 2 * ( op(1) == '<' );
    end
    tt = vr( : ) == 4;
    tt = tt( : );
    if any( tt ),
        x( tt ) = 0;
        y( tt ) = 1 - 2 * ( op(1) == '>' );
    end
end

%
% Detect redundancies and conflicts and act accordingly
%

if op(1) == '<',
    z = minus( y, x, cheat );
    op(1) = '>';
else
    z = minus( x, y, cheat );
end
temp = cvx_isconstant( z, true );
if any( temp( : ) ),
    zC = cvx_constant( z );
    switch op,
        case { '=', '==' }, t2 = temp & ( zC ~= 0 );
        case '~=', t2 = temp & ( zC == 0 );
        case '<=', t2 = temp & ( zC > 0  );
        case '<',  t2 = temp & ( zC >= 0 );
        case '>=', t2 = temp & ( zC < 0  );
        case '>',  t2 = temp & ( zC <= 0 );
    end
    if any( t2( : ) ),
        error( sprintf( 'Trivially infeasible constraints detected (e.g., 1 = 0, 2 > 3, etc.).\n   Trivially infeasible constraints are not permitted.\n    See the user guide for more details.' ) );
    else
        z( temp ) = 0;
    end
end
[ zR, zL, zS ] = bcompress( z );
if any( nonzeros( zR ) < 0 ) & op( 1 ) ~= '=',
    if length( op ) < 2,
        error( sprintf( 'Conflicting strict inequalities detected (e.g., a < b & b < a )\n   Trivially infeasible constraints are not permitted.\n   See the user guide for more details.' ) );
    else
        error( sprintf( 'Conflicting non-strict inequalities detected (e.g, a <= b & b <= a )\n   These must be converted to equalities in order to proceed.\n   See the user guide for more details.' ) );
    end
    return
end
if sdp_mode,
    if isreal( zR ),
        nnq = 0.5 * sz( 1 ) * ( sz( 1 ) + 1 );
    else
        nnq = sz( 1 ) * sz( 1 );
    end
    if size( zR, 1 ) > nnq * prod( sz( 3 : end ) ),
        warning( sprintf( [ ...
            'This linear matrix inequality appears to be unsymmetric. This is\n', ...
            'very likely an error that will produce unexpected results. Please check\n', ...
            'the LMI; and, if necessary, re-enter the model. (To disable this warning\n', ...
            'message, run the command ''cvx_sdp_warning(false)''.)' ] ) );
    end
end

%
% Add the (in)equalities
%

touch( prob, zL, op(1) == '=' );
mO = length( cvx___.equalities );
mN = length( zL );
cvx___.equalities = [ cvx___.equalities ; zL ];
cvx___.needslack( end + 1 : end + mN, : ) = op( 1 ) ~= '=';

%
% Create the dual
%

if ~isempty( dx ),
    zI = cvx_invert_structure( zR )';
    zI = sparse( mO + 1 : mO + mN, 1 : mN, 1 ) * zI;
    zI = cvx( zS, zI );
    duals = cvx___.problems( p ).duals;
    duals = builtin( 'subsasgn', duals, dx, zI );
    cvx___.problems( p ).duals = duals;
end

% Copyright 2005 Michael C. Grant and Stephen P. Boyd.
% See the file COPYING.txt for full copyright information.
% The command 'cvx_where' will show where this file is located.
