function [ x, y, status ] = cvx_solve_sedumi( At, b, c, sgn, nonls, quiet, prec )

[ n, m ] = size( At );
K.f = 0; K.l = 0; K.a = 0; K.q = []; K.s = [];
reord.f = []; reord.l = []; reord.q = [];
reord.ar = []; reord.ac = []; reord.av = [];
reord.sr = []; reord.sc = []; reord.sv = [];
K.s = []; K.scomplex = []; K.ycomplex = [];
for k = 1 : length( nonls ),
    temp = nonls( k ).indices;
    nn = size( temp, 1 );
    nv = size( temp, 2 );
    tt = nonls( k ).type;
    switch tt,
        case 'nonnegative',
            K.l = K.l + nn * nv;
            reord.l = [ reord.l ; temp( : ) ];
        case 'lorentz',
            if nn > 2,
                K.q = [ K.q, nn * ones( 1, nv ) ];
                temp = temp( [ end, 1 : end - 1 ], : );
                reord.q = [ reord.q ; temp( : ) ];
            else
                K.a  = K.a + nn * nv;
                stri = cvx_replicate_structure( [ 1, -1 ; 1, 1 ], nv );
                [ rr, cc, vv ] = find( stri );
                if ~isempty( reord.ac ),
                    cc = cc + reord.ac( end );
                end
                rr = temp( rr );
                reord.ar = [ reord.ar; rr( : ) ];
                reord.ac = [ reord.ac; cc( : ) ];
                reord.av = [ reord.av; vv( : ) ];
            end
        case { 'semidefinite', 'hermitian-semidefinite' },
            if tt( 1 ) == 'h',
                K.scomplex = [ K.scomplex, length( K.s ) + ( 1 : nv ) ];
                nn = sqrt( nn );
                str = cvx_create_structure( [ nn, nn, nv ], 'hermitian' );
            else
                nn = 0.5 * ( sqrt( 8 * nn + 1 ) - 1 );
                str = cvx_create_structure( [ nn, nn, nv ], 'symmetric' );
            end
            K.s = [ K.s, nn * ones( 1, nv ) ];
            stri = cvx_invert_structure( str, 'compact' )';
            [ rr, cc, vv ] = find( stri );
            if ~isempty( reord.sc ),
                cc = cc + reord.sc( end );
            end
            rr = temp( rr );
            reord.sr = [ reord.sr; rr( : ) ];
            reord.sc = [ reord.sc; cc( : ) ];
            reord.sv = [ reord.sv; vv( : ) ];
        otherwise,
            error( sprintf( 'Nonlinearity "%s" not supported for SeDuMi', tt ) );
    end
end

reord.f = 1 : n;
reord.f( [ reord.l ; reord.ar ; reord.q ; reord.sr ] ) = [];
K.f = length( reord.f );
g1 = K.f + K.l;
g3 = sum( K.q );
g2 = K.f + K.l + K.a + g3;
reord.sr = [ reord.f(:); reord.l; reord.ar;    reord.q;       reord.sr    ];
reord.sc = [ [1:g1]';             reord.ac+g1; [g2-g3+1:g2]'; reord.sc+g2 ];
reord.sv = [ ones(g1,1);          reord.av;    ones(g3,1);    reord.sv    ];
reord = sparse( reord.sr, reord.sc, reord.sv );

At = reord' * At;
c = reord' * c;
nn = size( At, 1 );
K.l = K.l + K.a;
pars.eps = min(prec);
pars.bigeps = max(prec);
if quiet,
    pars.fid = 0;
end
pars.free = 0; % ~isempty( K.q ) | ~isempty( K.s );
pars.sdp = 0;

if ~quiet,
    disp( ' ' );
    disp( sprintf( 'Calling SeDuMi: %d variables (%d free), %d equality constraints', n, K.f, m ) );
    disp( '------------------------------------------------------------------------' );
end

degen = 0;
if K.f == size( At, 1 ) | length( c ) <= length( b ),
    degen = max( length( b ) - length( c ) + 1, 1 );
    At = [ At( 1 : K.f, : ) ; sparse( degen, size(At,2) ) ; At( K.f + 1 : end, : ) ];
    c  = [  c( 1 : K.f, : ) ; zeros(  degen, 1          ) ;  c( K.f + 1 : end, : ) ];
    K.l = K.l + 1;
end
global cvx___
if cvx___.profile,
    profile off
end
[ x, y, info ] = sedumi( At, b, c, K, pars );
if cvx___.profile,
    profile resume
end
x = full( x );
y = full( y );
if degen,
    x( K.f + 1 : K.f + degen ) = [];
end
if info.numerr == 2,
    status = 'Failed';
    x = NaN * ones( nn, 1 );
    y = NaN * ones( m, 1 );
elseif info.pinf ~= 0,
    status = 'Infeasible';
    x = NaN * ones( nn, 1 );
elseif info.dinf ~= 0,
    status = 'Unbounded';
    y = NaN * ones( m, 1 );
else
    status = 'Solved';
end
if info.numerr == 1 & pars.eps > 0,
    status = [ 'Inaccurate/', status ];
end

x = real( reord * x );

% Copyright 2005 Michael C. Grant and Stephen P. Boyd.
% See the file COPYING.txt for full copyright information.
% The command 'cvx_where' will show where this file is located.
