program Veltran                       ! Velocity transform
  use sep
  use velsimpx
!  use solver_mod
  use solver_smp_mod
  use solver_reg_mod
  use cgstep_mod
!	use scaled_identity
	use weight
	use weight2                 ! duplicate

  implicit none

  integer                          :: nt, nx, ns, n3, iadj, iinv, ireg, &
                                      ilint, n2, niter, stat, i3, ilast
  real                             :: o1,d1, o2,d2, x0,dx, s0,ds, eps
  real, dimension(:), allocatable  :: data, model, m0
  real, dimension(:), pointer      :: modwt, reswt
  logical                          :: adj, inv, lint, reg, uselast

  call sep_init()
  call from_history( "n1", nt       )
  call from_history( "n2", n2       )
  call from_history( "o1", o1       )
  call from_history( "d1", d1       )
  call from_history( "n3", n3       )
  call from_either(  "nx", nx, n2   )
  call from_either(  "ns", ns, 2*nx )

  call from_param( "adj",  iadj, 1);   adj  = (iadj==1)
  call from_param( "inv",  iinv, 0);   inv  = (iinv==1)
  call from_param( "reg",  ireg, 0);   reg  = (ireg==1)
  call from_param( "lint", ilint, 0);  lint = (ilint==1)

  if (adj) then                      ! Input is CMP data
     call from_history( "o2", x0 )
     call from_history( "d2", dx )
     call from_param( "s0", s0 )
     call from_param( "ds", ds )
     call to_history ("label2","slowness(sec/km)")
     n2 = ns; o2 = s0; d2 = ds
  else                               ! Input is slowness scan
     call from_history( "o2", s0 )
     call from_history( "d2", ds )
     call from_param( "x0", x0 )
     call from_param( "dx", dx )
     call to_history ("label2","distance(km)")
     n2 = nx; o2 = x0; d2 = dx
  end if
  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)
  call to_history ("n3", n3)

	if( inv ) then
  	call from_param( "uselast", ilast, 1);  uselast = (ilast==1)
		allocate( m0(nt*ns) );  m0=0.0
		if( reg) then
			call from_param( "eps", eps, 1.0 )
			allocate( modwt(nt*ns) )
			modwt=1.0
		end if
		allocate( reswt(nt*nx) )
		reswt=1.0
	end if

  call from_param( "niter", niter, 1)
!  call sep_close()

  allocate( data(nt*nx), model(nt*ns) )

	do i3=1,n3
		write(0,*) 'CMP ',i3,' of ',n3

		call velsimpx_init( o1, d1, x0, dx, s0, ds, nt, nx, ns, lint)
		if( adj) then                    ! Adjoint or LS inverse
			call sep_read( data)
			if( inv) then		                 ! LS inverse...	
				if( exist_file("reswt")) call sep_read( reswt, "reswt")
				call weight2_init( reswt)
				if( reg) then	                   ! Damped LS
					if( exist_file("modwt")) call sep_read( modwt, "modwt")
					call weight_init( modwt)
					call solver_reg( model, data, velsimpx_lop, weight_lop, cgstep, &
                           nt*ns, niter, eps, verb=.true., Wop=weight2_lop, m0=m0)
				else                             ! Normal LS
					call solver_smp( model, data, velsimpx_lop, cgstep, niter, &
                           verb=.true., Wop=weight2_lop, m0=m0)
				end if
				if( uselast) m0=model
			else                             ! Adjoint.
				stat = velsimpx_lop( adj, .false., model, data)
			end if
			call sep_write( model)
		else                             ! Forward Modeling
			call sep_read( model)
			stat = velsimpx_lop( adj, .false., model, data)
			call sep_write( data)
		end if
	end do

  deallocate( model, data)
	if(allocated(reswt)) deallocate(reswt)
	if(allocated(modwt)) deallocate(modwt)

  call exit (0)
end program Veltran
