# 1 "<stdin>"
# 1 "<built-in>"
# 1 "<command line>"
# 1 "<stdin>"
!$=head1 NAME
!$
!$igrad1 - 1D gradient operator
!$
!$=head1 SYNOPSIS
!$
!$C<ierr=igrad1_lop(adj,add,xx,yy)>
!$
!$=head1 PARAMETERS
!$
!$=over 4
!$
!$=item adj,add,xx,yy -
!$
!$ standard operator interface
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Calculate 1-D gradient
!$
!$=head1 SEE ALSO
!$
!$L<igrad2>
!$
!$=head1 LIBRARY
!$
!$B<geef90>
!$
!$=cut
!$
module igradt
! gradient in one dimension
  use adj_mod
  implicit none
  integer, private, save ::n1,n2
  contains
  function igradt_lop ( adj, add, xx, yy,n1,n2) result(stat)
    integer :: stat
    logical,intent(in) :: adj,add
    integer, intent(in)   ::n1,n2
    real,dimension(:) :: xx,yy
    call adjnull (adj,add,xx,yy )
    call igradt_lop2(adj,add,xx,yy,n1,n2 )
    stat=0
  end function igradt_lop
  subroutine igradt_lop2(adj,add,xx,yy,n1,n2)
    logical,intent(in) :: adj,add
    integer,intent(in) :: n1,n2
    real, dimension (:) :: xx,yy
    real, dimension (:,:),pointer :: model,data
    integer                     :: stat, i, j
    !integer i
    allocate( model(n1,n2), data(n1,n2) )
    model = reshape(xx,(/n1,n2/))
    data = reshape(yy,(/n1,n2/))
    do i=1,n1-1  
       do j=1,n2
          if ( adj) then
             model(i+1,j)= (model(i+1,j)+data(i,j))
             model(i,j)  = (model(i,  j)-data(i,j))
          else
             data(i,j)  = data(i,j)+(model(i+1,j)-model(i,j))
          end if
       end do
    end do
    do j=1,n2
       if(adj) then
          model(n1,j)=model(n1-1,j)
       else
          data(n1,j)=data(n1-1,j)
       end if
    end do
    !do i= 1, size(xx)-1
    !  if ( adj) then
    !    xx(i+1) = xx(i+1) + yy(i) ! resembles equation (1.2)
    !    xx(i ) = xx(i ) - yy(i)
    !  else
    !    yy(i) = yy(i) + xx(i+1) - xx(i) ! resembles equation (1.1)
    !  end if
    !end do
    xx = reshape(model,(/n1*n2/))
    yy = reshape(data,(/n1*n2/))
  end subroutine igradt_lop2
  !subroutine igrad1a_close()
  !end subroutine igrad1a_close
end module igradt
