# 1 "<stdin>"
# 1 "<built-in>"
# 1 "<command line>"
# 1 "<stdin>"
!$=head1 NAME
!$
!$igrad1 - 1D gradient operator
!$
!$=head1 SYNOPSIS
!$
!$C<ierr=igrad1_lop(adj,add,xx,yy)>
!$
!$=head1 PARAMETERS
!$
!$=over 4
!$
!$=item adj,add,xx,yy -
!$
!$ standard operator interface
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Calculate 1-D gradient
!$
!$=head1 SEE ALSO
!$
!$L<igrad2>
!$
!$=head1 LIBRARY
!$
!$B<geef90>
!$
!$=cut
!$
module igradx
! gradient in one dimension
  use adj_mod
  implicit none
  integer, private, save ::n1,n2
  contains
  function igradx_lop ( adj, add, xx, yy,n1,n2) result(stat)
    integer :: stat
    logical,intent(in) :: adj,add
    integer, intent(in)   ::n1,n2
    real,dimension(:) :: xx,yy
    call adjnull (adj,add,xx,yy )
    call igradx_lop2(adj,add,xx,yy,n1,n2 )
    stat=0
  end function igradx_lop
  subroutine igradx_lop2(adj,add,xx,yy,n1,n2)
    logical,intent(in) :: adj,add
    integer,intent(in) :: n1,n2
    real, dimension (:) :: xx,yy
    real, dimension (:,:),pointer :: model,data
    integer                     :: stat, i, j
    
    allocate( model(n1,n2), data(n1,n2) )
    model = reshape(xx,(/n1,n2/))
    data = reshape(yy,(/n1,n2/))
    !write(0,*) 'mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm'
    !write(0,*) shape(model)
    !write(0,*) shape(data)
    !write(0,*) size(xx)
    !write(0,*) size(yy)
    do i=1,n1  
       do j=1,n2-1
          !write(0,*) i, j
          if ( adj) then
             model(i,j+1)= (model(i,j+1)+data(i,j))
             model(i,j)  = (model(i,j  )-data(i,j))
          else
             data(i,j)  = data(i,j)+(model(i,j+1)-model(i,j))
          end if
       end do
    end do
    do i=1,n1  
       if(adj) then
          model(i,n2)=model(i,n2-1)
       else
          data(i,n2)=data(i,n2-1)
       end if
    end do
    !do i= 1, size(xx)-1
    !  if ( adj) then
    !    xx(i+1) = xx(i+1) + yy(i) ! resembles equation (1.2)
    !    xx(i ) = xx(i ) - yy(i)
    !  else
    !    yy(i) = yy(i) + xx(i+1) - xx(i) ! resembles equation (1.1)
    !  end if
    !end do
    xx = reshape(model,(/n1*n2/))
    yy = reshape(data,(/n1*n2/))
  end subroutine igradx_lop2
end module igradx
