module gradchain_2fit_mod
  use chain_mod
  use igrad1
  use adj_mod
  implicit none
  integer, private, save                     :: n1, n2
  real, private, save                        :: epsx,epst
  real, dimension(:), private, save, pointer :: wtx,wtt
  contains
!---------------------------------------------------------------------------------
  subroutine gradchain2d_init(wtx_in,wtt_in, n1_in, n2_in, epsx_in,&
    & epst_in)
    integer                    :: n1_in, n2_in
    real                       :: epsx_in,epst_in
    real, dimension(:), pointer :: wtx_in, wtt_in
    allocate( wtx(size(wtx_in)),wtt(size(wtt_in))  )
    wtx = wtx_in 
    wtt = wtt_in                             
    n1 = n1_in
    n2 = n2_in
    epsx = epsx_in
    epst = epst_in
  end subroutine
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function diagweightx_lop( adj, add, model, data)
    real, dimension(:)            :: model, data
    logical                       :: adj,add
    call adjnull( adj, add, model, data)
    if (adj) then
      model = model +epsx*wtx*data
    else
      data  = data  +epsx*wtx*model
    end if 
    diagweightx_lop = 0
  end function
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function diagweightt_lop( adj, add, model, data)
    real, dimension(:)            :: model, data
    logical                       :: adj,add
    call adjnull( adj, add, model, data)
    if (adj) then
      model = model +epst*wtt*data
    else
      data  = data  +epst*wtt*model
    end if 
    diagweightt_lop = 0
  end function
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function weighted_gradx_lop( adj, add, model, data)
    logical            :: adj,add
    real, dimension(:) :: model, data
    real, pointer      :: temp(:)
    allocate( temp(size(model)) )
    call chain( diagweightx_lop, xxgrad1_lop, adj, add, model, data,&
      & temp )
    deallocate( temp)
    weighted_gradx_lop = 0
  end function
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function weighted_gradt_lop( adj, add, model, data)
    logical            :: adj,add
    real, dimension(:) :: model, data
    real, pointer      :: temp(:)
    allocate( temp(size(model)) )
    call chain( diagweightt_lop, ttgrad1_lop, adj, add, model, data,&
      & temp )
    deallocate( temp)
    weighted_gradt_lop = 0
  end function
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function ttgrad1_lop( adj, add, model, data)
    logical                       :: adj,add
    real, dimension(:)            :: model, data
    real, dimension(:,:), pointer :: model2d, data2d
    integer                       :: stat, i1, i2
    call adjnull( adj, add, model, data)
    allocate( model2d(n1,n2), data2d(n1,n2) )
    model2d = reshape(model,(/n1,n2/))
    data2d = reshape(data,(/n1,n2/))
    do i1=1,n1  
      if ( adj) then
        stat = igrad1_lop( .true., add, model2d(i1,:), data2d(i1,:) )
      else
        stat = igrad1_lop( .false., add, model2d(i1,:), data2d(i1,:) )
      end if
    end do 
    model = reshape(model2d,(/n1*n2/))
    data  = reshape(data2d,(/n1*n2/))
    deallocate( model2d, data2d)
    ttgrad1_lop = 0
  end function
!---------------------------------------------------------------------------------
!---------------------------------------------------------------------------------
  integer function xxgrad1_lop( adj, add, model, data)
    logical                       :: adj,add
    real, dimension(:)            :: model, data
    real, dimension(:,:), pointer :: model2d, data2d
    integer                       :: stat, i1, i2
    call adjnull( adj, add, model, data)
    allocate( model2d(n1,n2), data2d(n1,n2) )
    model2d = reshape(model,(/n1,n2/))
    data2d = reshape(data,(/n1,n2/))
    do i2=1,n2  
      if ( adj) then
        stat = igrad1_lop( .true., add, model2d(:,i2), data2d(:,i2) )
      else
        stat = igrad1_lop( .false., add, model2d(:,i2), data2d(:,i2) )
      end if
    end do 
    model = reshape(model2d,(/n1*n2/))
    data = reshape(data2d,(/n1*n2/))
    deallocate( model2d, data2d)
    xxgrad1_lop = 0
  end function
!---------------------------------------------------------------------------------
end module gradchain_2fit_mod
