module vrms2int_2d_2fit  
! Transform from RMS to interval velocity
  use causint1
  use cgstep_mod
  use solver_mod
  use gradchain_2fit_mod
  use scaled_identity
  use fit2_mod
  use sep
  use dottest
implicit none
  contains
! --------------------------------------------------------------------------------
  subroutine vrms2int_2d_2fit_exe( niter, eps,epsx,epst, weight, vrms,&
    & vint, nq,alphax,alphat,n1,n2,dat,qx,qz)
    integer,            intent( in)      :: niter, nq 
    ! iterations(linear, cauchy)
    integer,            intent( in)      :: n1, n2  
    ! model and data dimensions
    real,               intent( in)      :: eps,epsx,epst,alphax,alphat
! scaling parameter
    real, dimension(:), intent( in )     :: vrms      
    ! RMS velocity
    real, dimension(:), intent( out)     :: vint      
    ! interval velocity
    real, dimension(:), pointer          :: weight    
    ! data weighting
    real, dimension(:), pointer          :: qx,qz,qqx,qqz
    integer                              :: stat,it,ii,ix,iq,n
    logical, dimension( size( vint))     :: mask
    real,    dimension( size( vrms))     :: dat,resi 
    real,    dimension( 2)               :: dot1,dot2 
    call sep_init ()
! --------------------------------------------------------------------------------
    n = n1*n2
! --------------------------------------------------------------------------------
    do ix= 1, n2  
      do it= 1, n1  
        dat( it+(ix-1)*n1) = vrms( it+(ix-1)*n1) * vrms( it+(ix-1)*n1)&
          & * it
!  weight( it+(ix-1)*n1) = weight( it+(ix-1)*n1)*(1./it)      # decrease weight with time
        weight( it+(ix-1)*n1) = (12./it)      
        ! decrease weight with time
      end do
    end do
! --------------------------------------------------------------------------------
    mask = .false.
    vint = 0.     
    do ix= 1, n2  
      mask(1 + (ix-1)*n1 ) = .true.   
      vint(1 + (ix-1)*n1 ) = dat(1 + (ix-1)*n1 )
    end do
! --------------------------------------------------------------------------------
    allocate( qqx(n),qqz(n))
    do iq= 1, nq  
      write(0,*) "iteration Q",iq
      if (iq .eq. 1 ) then
        qx=1.
        qz=1.
      else
        write(0,*)"!!!!!!!!!!!!!!!!!!!!!!!!"
        stat = xxgrad1_lop(.false.,.false.,vint,qqx) 
        stat = ttgrad1_lop(.false.,.false.,vint,qqz) 
        qx=1./(sqrt(1+(qqx/alphax)**2))  
        qz=1./(sqrt(1+(qqz/alphat)**2))
        write(0,*)"minval(model),maxval(model)", minval(vint),maxval&
          &(vint)
        write(0,*)"minval(qqx),maxval(qqx)", minval(abs(qqx)),maxval(abs&
          &(qqx)),alphax
        write(0,*)"minval(qqz),maxval(qqz)", minval(abs(qqz)),maxval(abs&
          &(qqz)),alphat
        write(0,*)"minval(qx),maxval(qx) ", minval(qx),maxval(qx)
        write(0,*)"minval(qz),maxval(qz) ", minval(qz),maxval(qz)
      end if 
      write(0,*)"epsx,epst",epsx,epst
      call gradchain2d_init( qx,qz, n1, n2, epsx,epst)
      call init_array(n)
      call causint1_init(n1)
      call dot_test( my_array,n,2*n,dot1,dot2)
!do it=1, 20{
!    write(0,*) "weight",weight(it)*it
!   }
      call solver_reg( causint1_lop, cgstep, my_array  , 2*n, vint, dat&
        &, niter, eps,                        known=mask, x0=vint,&
        & wt=weight, verb=.true.,res=resi)
      resi = resi*weight 
      call cgstep_close()
    end do 
    if ( exist_file("qx")) then
      call sep_write( qx, "qx")
      call to_history ("n1",n1,"qx")
      call to_history ("n2",n2,"qx")
      call to_history ("n3",1,"qx")
    end if
    if ( exist_file("qz")) then
      call sep_write(qz , "qz")
      call to_history ("n1",n1,"qz")
      call to_history ("n2",n2,"qz")
      call to_history ("n3",1,"qz")
    end if
    if ( exist_file("res")) then
      call sep_write(resi , "res")
      call to_history ("n1",n1,"res")
      call to_history ("n2",n2,"res")
      call to_history ("n3",1,"res")
    end if
    deallocate( qqx,qqz)    
    vint = sqrt( vint)
! --------------------------------------------------------------------------------
  end subroutine
! --------------------------------------------------------------------------------
end module 
