module vrms2int_l1 {                # Transform from RMS to interval velocity
  use causint
  use igrad1
  use cgstep_mod
  use solver_mod
  use gradchain

contains
# --------------------------------------------------------------------------------
  subroutine vrms2int( niter, eps, weight, vrms, vint, prec, err, res,nq,alpha ) {
    integer,            intent( in)      :: niter, nq # iterations(linear, cauchy)
    real,               intent( in)      :: eps,alpha # scaling parameter
    real, dimension(:), intent( in out)  :: vrms      # RMS velocity    
    real, dimension(:), intent( out)     :: vint      # interval velocity
    real, dimension(:), intent( in out)  :: err       # residual error per iteration
    real, dimension(:), intent( in out)  :: res       # final residual error
    real, dimension(:), pointer          :: weight    # data weighting
    real, dimension(:), pointer          :: q         # Cauchy 
    integer                              :: st,it,nt,iq,it1  
    logical                              :: prec
    logical, dimension( size( vint))     :: mask
    real,    dimension( size( vrms))     :: dat 
# --------------------------------------------------------------------------------
    nt = size( vrms)
    do it= 1, nt {
       dat( it) = vrms( it) * vrms( it) * it
       weight( it) = weight( it)*(1./it)           # decrease weight with time 
#      weight( it) = (12./it)                       # decrease weight with time 
    }
# --------------------------------------------------------------------------------
    mask = .false.;   mask( 1) = .true.            # constrain first point
    vint = 0.     ;   vint( 1) = dat( 1)
    allocate (q(nt))
# --------------------------------------------------------------------------------
    do iq= 1, nq {
      if (iq == 1 ) {
         q=1
      }
      else {
         do it1= 1, nt-1 { 
            q(it1)=1./sqrt(1+(1./(alpha)**2)*(vint(it1+1)-vint(it1))**2)  
			write(0,*) "q",iq,q(it1),(1./(alpha)**2)*(vint(it1+1)-vint(it1))**2
         }
         q(nt)=q(nt-1)
      }
      
      if( prec) {
        call solver_prec( causint_lop, cgstep, x=vint, dat=dat, niter=niter, known=mask, 
                        x0=vint, wt=weight, prec=causint_lop, eps=eps, nprec=nt,
                        err=err, res=res )
        st = causint_lop( .false., .false., vint, dat)
      }
      else {
        call gradchain_init( q )

        call solver_reg( causint_lop, cgstep, weighted_grad_lop, nt, vint, dat, niter, eps,
                       known=mask, x0=vint, wt=weight, err=err, res=res , verb=.true.)
      do it= 1, nt {
      if (vint(it)< 2.25) {
      vint(it) = 2.25
      }
      }

	  }
    }
# --------------------------------------------------------------------------------
	res = res*weight
    call cgstep_close()

    do it= 1, nt {
       vrms( it) = sqrt( dat( it)/it)
       weight( it) = weight(it)*it
	}
    vint = sqrt( vint)
# --------------------------------------------------------------------------------
    }
# --------------------------------------------------------------------------------


}
